#include "jngen.h"

using ll = long long;

template<typename TI, typename TF>
class generic_point {
public:
    TI x, y;

    generic_point() = default;

    generic_point(TI _x, TI _y) : x(_x), y(_y) {}

    generic_point<TI, TF> operator+() const {
        return *this;
    }

    generic_point<TI, TF> operator-() const {
        return generic_point<TI, TF>(-x, -y);
    }

    generic_point<TI, TF> operator+(const generic_point<TI, TF>& other) const {
        return generic_point<TI, TF>(x + other.x, y + other.y);
    }

    generic_point<TI, TF> operator-(const generic_point<TI, TF>& other) const {
        return generic_point<TI, TF>(x - other.x, y - other.y);
    }

    generic_point<TI, TF> operator*(TI k) const {
        return generic_point<TI, TF>(x * k, y * k);
    }

    generic_point<TI, TF>& operator+=(const generic_point<TI, TF>& other) {
        *this = (*this) + other;
        return *this;
    }

    generic_point<TI, TF>& operator-=(const generic_point<TI, TF>& other) {
        *this = (*this) - other;
        return *this;
    }

    generic_point<TI, TF>& operator*=(TI k) {
        *this = (*this) * k;
        return *this;
    }

    template<typename T = TI>
    typename std::enable_if<std::is_floating_point<T>::value, generic_point<TI, TF>>::type
    operator/(TI k) const {
        return generic_point<TI, TF>(x / k, y / k);
    }

    template<typename T = TI>
    typename std::enable_if<std::is_floating_point<T>::value, generic_point<TI, TF>&>::type
    operator/=(TI k) {
        *this = (*this) / k;
        return *this;
    }

    TI operator*(const generic_point<TI, TF>& other) const {
        return x * other.x + y * other.y;
    }

    TI operator%(const generic_point<TI, TF>& other) const {
        return x * other.y - y * other.x;
    }

    TI lenSqr() const {
        return x * x + y * y;
    }

    TF len() const {
        return sqrt(lenSqr());
    }

    TI distSqr(const generic_point<TI, TF>& other) const {
        return (other - *this).lenSqr();
    }

    TF dist(const generic_point<TI, TF>& other) const {
        return sqrt(distSqr(other));
    }

    generic_point<TI, TF> norm() const {
        return *this / len();
    }

    // Counterclockwise rotation by 90 degrees
    generic_point<TI, TF> ort() const {
        return generic_point<TI, TF>(-y, x);
    }

    // Counterclockwise rotation by c = cos(alpha), s = sin(alpha)
    template<typename T = TI>
    typename std::enable_if<std::is_floating_point<T>::value, generic_point<TI, TF>>::type
    rot(TF c, TF s) const {
        return *this * c + ort() * s;
    }

    // Counterclockwise rotation by alpha
    generic_point<TI, TF> rot(TF alpha) const {
        return rot(cos(alpha), sin(alpha));
    }

    TF arg() const {
        return atan2(y, x);
    }

    int quad() const {
        if (x >= 0 && y >= 0) {
            return 1;
        } else if (x < 0 && y >= 0) {
            return 2;
        } else if (x < 0 && y < 0) {
            return 3;
        } else if (x >= 0 && y < 0) {
            return 4;
        }
        throw;
    }

    template<typename T = TI>
    typename std::enable_if<std::is_floating_point<T>::value>::type
    scan() {
        double _x, _y;
        scanf("%lf%lf", &_x, &_y);
        x = _x;
        y = _y;
    }

    template<typename T = TI>
    typename std::enable_if<std::is_integral<T>::value>::type
    scan() {
        ll _x, _y;
        scanf("%lld%lld", &_x, &_y);
        x = _x;
        y = _y;
    }

    template<typename T = TI>
    typename std::enable_if<std::is_floating_point<T>::value>::type
    print() {
        printf("%.10lf %.10lf\n", (double)x, (double)y);
    }

    template<typename T = TI>
    typename std::enable_if<std::is_integral<T>::value>::type
    print() {
        printf("%lld %lld\n", (ll)x, (ll)y);
    }
};

using point = generic_point<ll, double>;

const int N = 2010;

point a[N];
int g[N * N];
std::vector<int> gr[N * N];
int position[N];
int type[N]; // 1 - green, 2 - blue, 3 - red
int used[N * N], par[N * N];
bool inCycle[N * N];
int tin[N * N], tOut[N * N];
int timer = 0;

void findCycle(int v) {
    used[v] = 1;

    int to = g[v];
    if (to != -1) {
        if (used[to] == 0) {
            par[to] = v;
            findCycle(to);
        } else if (used[to] == 1 && !inCycle[to]) {
            for (int i = v; i != to; i = par[i]) {
                inCycle[i] = true;
            }
            inCycle[to] = true;
        }
    }

    used[v] = 2;
}

void dfs(int v) {
    used[v] = 1;
    tin[v] = timer++;

    for (auto to : gr[v]) {
        if (!inCycle[to] && used[to] == 0) {
            dfs(to);
        }
    }

    tOut[v] = timer++;
}

bool isParent(int v1, int v2) {
    return tin[v1] <= tin[v2] && tOut[v1] >= tOut[v2];
}

bool solve(int n, const std::vector<std::pair<int, int>>& allPoints, int k) {
	for (int i = 0; i < n * n; ++i) {
		g[i] = -1;
		gr[i].clear();
		used[i] = 0;
		par[i] = 0;
		inCycle[i] = false;
		tin[i] = 0;
		tOut[i] = 0;
	}
	timer = 0;

    for (int i = 0; i < n; ++i) {
    	a[i] = point(allPoints[i].first, allPoints[i].second);
    }

    std::vector<std::pair<point, int>> points;
    for (int j = 0; j < n; ++j) {
        points.clear();
        for (int i = 0; i < n; ++i) {
            if (i == j) {
                continue;
            }

            points.emplace_back(a[i] - a[j], i);
        }

        std::sort(points.begin(), points.end(), [&](const std::pair<point, int>& p1, const std::pair<point, int>& p2) {
            const auto& pt1 = p1.first;
            const auto& pt2 = p2.first;

            if (pt1.quad() != pt2.quad()) {
                return pt1.quad() < pt2.quad();
            }

            ll cross = pt1 % pt2;
            if (cross != 0) {
                return cross > 0;
            } else {
                return pt1.lenSqr() < pt2.lenSqr();
            }
        });

        for (int i = 0; i < n; ++i) {
            if (i == j) {
                continue;
            }

            auto pos = std::lower_bound(points.begin(), points.end(), std::make_pair(a[j] - a[i], -1), [&](const std::pair<point, int>& p1, const std::pair<point, int>& p2) {
                const auto& pt1 = p1.first;
                const auto& pt2 = p2.first;

                if (pt1.quad() != pt2.quad()) {
                    return pt1.quad() < pt2.quad();
                }

                ll cross = pt1 % pt2;
                if (cross != 0) {
                    return cross > 0;
                } else {
                    if (p1.second == -1) {
                        return true;
                    } else if (p2.second == -1) {
                        return false;
                    } else {
                        return pt1.lenSqr() < pt2.lenSqr();
                    }
                }
            }) - points.begin();

            int vertexId = i * n + j;
            int nextPos = (pos + k) % (n - 1);
            int nextVertexId = j * n + points[nextPos].second;

            g[vertexId] = nextVertexId;
            gr[nextVertexId].push_back(vertexId);
        }
    }

    for (int i = 0; i < n; ++i) {
        type[i] = 3;
    }

    for (int i = 0; i < n * n; ++i) {
        if (used[i] == 0) {
            findCycle(i);
        }
    }
    for (int i = 0; i < n * n; ++i) {
    	used[i] = 0;
    }
    for (int i = 0; i < n * n; ++i) {
        if (!inCycle[i] && used[i] == 0) {
            dfs(i);
        }
    }

    for (int i = 0; i < n * n; ++i) {
        if (!inCycle[i]) {
            continue;
        }

        int from = i / n;
        int to = i % n;
        if (from == to) {
            continue;
        }

        type[from] = 1;
        type[to] = 1;
    }

    std::vector<int> curVertices;
    for (int i = 0; i < n; ++i) {
        if (type[i] == 1) {
            continue;
        }

        curVertices.clear();
        for (int j = 0; j < n; ++j) {
            if (i == j) {
                continue;
            }
            curVertices.push_back(i * n + j);
            curVertices.push_back(j * n + i);
        }

        std::sort(curVertices.begin(), curVertices.end(), [&](int v1, int v2) {
            return tin[v1] < tin[v2];
        });

        for (int j = 1; j < (int)curVertices.size(); ++j) {
            int v1 = curVertices[j - 1];
            int v2 = curVertices[j];
            
            if (isParent(v1, v2) || isParent(v2, v1)) {
                type[i] = 2;
                break;
            }
        }
    }

    for (int i = 0; i < n; ++i) {
    	if (type[i] == 2) {
    		return false;
    	}
    }
    return true;
}

int main(int argc, char** argv) {
	registerGen(argc, argv, 1);

	int n = atoi(argv[1]);
    int maxCoord = atoi(argv[2]);
    int noiseCnt = atoi(argv[3]);

    auto p = rndg.convexPolygon(n - noiseCnt, -maxCoord, maxCoord);
    std::set<std::pair<int, int>> setik;
    for (int i = 0; i < n - noiseCnt; ++i) {
    	setik.emplace(p[i].x, p[i].y);
    }
    while ((int)setik.size() != n) {
    	int x = rnd.next(-maxCoord, maxCoord);
    	int y = rnd.next(-maxCoord, maxCoord);
    	setik.emplace(x, y);
    }
	std::vector<std::pair<int, int>> points(setik.begin(), setik.end());

	do {
	    int k = rnd.next(0, n - 2);
		if (solve(n, points, k)) {
			std::cout << n << ' ' << k + 1 << std::endl;
			for (auto[x, y] : points) {
				std::cout << x << ' ' << y << std::endl;
			}
			break;
		}
	} while (std::next_permutation(points.begin(), points.end()));

	return 0;
}